'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.mainReducer = mainReducer;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _browserReducer = require('./browserReducer');

var _subscribersReducer = require('./subscribersReducer');

var _subscriptionsReducer = require('./subscriptionsReducer');

var _isBrowserProcess = require('../utils/isBrowserProcess');

var _logToFile = require('../utils/logToFile');

var _actionTypes = require('../constants/actionTypes');

var CONSTANTS = _interopRequireWildcard(_actionTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const initialState = _immutable2.default.fromJS({
	options: {
		windowName: ''
	}
});

function mainReducer() {
	let state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
	let action = arguments[1];


	switch (action.type) {
		// Debugger
		case CONSTANTS.GLOBAL_ACTION:
			{
				if (state.getIn(['options', 'debug', 'enabled']) === true) {
					state = state.updateIn(['debugger', 'globalActions'], _immutable2.default.List(), actions => {
						return actions.push(action).slice(state.getIn(['options', 'debug', 'numOfActionsToStore'], 15) * -1);
					});

					if ((0, _isBrowserProcess.isBrowserProcess)()) {
						(0, _logToFile.logToFile)('[Global Action]: ' + action.payload.action.type);
					}
				}
				break;
			}
	}

	switch (action.type) {
		case CONSTANTS.ANNOUNCE_NEW_VIEW:
		case CONSTANTS.ANNOUNCE_CLOSING_VIEW:
		case CONSTANTS.ADD_GLOBAL_ACTION_TO_QUEUE:
		case CONSTANTS.CLEAR_ACTION_QUEUE_FOR_VIEW:
		case CONSTANTS.ADD_GLOBAL_ACTION_TO_STORE:
		case CONSTANTS.REMOVE_VIEW_FROM_ACTION_STORE:
			{
				if ((0, _isBrowserProcess.isBrowserProcess)()) {
					state = (0, _browserReducer.browserReducer)(state, action);
				}
			}
	}

	switch (action.type) {
		case CONSTANTS.ADD_ACTION_SUBSCRIBER:
		case CONSTANTS.REMOVE_ACTION_SUBSCRIBER:
		case CONSTANTS.ADD_STATE_SUBSCRIBER:
		case CONSTANTS.REMOVE_STATE_SUBSCRIBER:
		case CONSTANTS.CLEAR_SUBSCRIPTIONS_FOR_WINDOW:
		case CONSTANTS.ANNOUNCE_NEW_VIEW:
		case CONSTANTS.ANNOUNCE_CLOSING_VIEW:
			{
				state = (0, _subscribersReducer.subscribersReducer)(state, action);
			}
	}

	switch (action.type) {
		case CONSTANTS.SUBSCRIBE_TO_STATE:
		case CONSTANTS.UNSUBSCRIBE_FROM_STATE:
		case CONSTANTS.SUBSCRIBE_TO_ACTIONS:
		case CONSTANTS.UNSUBSCRIBE_FROM_ACTIONS:
		case CONSTANTS.STATE_SUBSCRIPTION_STATUS:
		case CONSTANTS.ACTIONS_SUBSCRIPTION_STATUS:
		case CONSTANTS.STATE_SUBSCRIPTION_REQUEST_SENT:
		case CONSTANTS.ACTIONS_SUBSCRIPTION_REQUEST_SENT:
		case CONSTANTS.CLEAR_SUBSCRIPTIONS_FOR_WINDOW:
		case CONSTANTS.ANNOUNCE_NEW_VIEW:
		case CONSTANTS.ANNOUNCE_CLOSING_VIEW:
			{
				state = (0, _subscriptionsReducer.subscriptionsReducer)(state, action);
			}
	}

	return state;
}