'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.stateChangeReducer = stateChangeReducer;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _actionTypes = require('../constants/actionTypes');

var _utils = require('../utils');

var _selectors = require('../selectors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function stateChangeReducer(state, action) {
	switch (action.type) {
		case _actionTypes.STATE_UPDATE:
			{
				// TODO: Check if we are subscribed and if we have a state filter for this
				const stateUpdate = action.payload.stateUpdate;
				return (0, _utils.patchStateFromDiff)(state, stateUpdate);
			}

		case _actionTypes.SET_INITIAL_SUBSCRIBED_STATE:
			{
				var _action$payload = action.payload;
				const initialState = _action$payload.initialState;
				const stateFrom = _action$payload.stateFrom;

				const subscribeTo = (0, _selectors.getSubscribeToByViewName)(state, { viewName: stateFrom });

				var shape = subscribeTo.get('stateFilter', _immutable2.default.Map());

				if (!_immutable2.default.Map.isMap(shape) || shape.size <= 0) {
					console.warn(`Received initial state from view "${ stateFrom }", but there is no known filter for that state`);

					return state;
				} else {
					return (0, _utils.mergeStateWithShape)(state, initialState, shape);
				}
			}

		default:
			{
				return state;
			}
	}
}