'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.subscribersReducer = subscribersReducer;
exports.clearSubscriptionsFromWindow = clearSubscriptionsFromWindow;
exports.clearActionSubscriptionFromWindow = clearActionSubscriptionFromWindow;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _calculateStateFilterShape = require('../utils/calculateStateFilterShape');

var _actionTypes = require('../constants/actionTypes');

var CONSTANTS = _interopRequireWildcard(_actionTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const initialState = _immutable2.default.fromJS({
	// 		stateSubscribers: {
	// 			windowName: {
	// 				windowName: 'windowName',
	// 				filter    : 'filter'
	// 			}
	// 		},
	//
	// 		stateFilterShape: {},
	//
	// 		actionSubscribers: {
	// 			action: ['windowName']
	// 		},
});

function subscribersReducer() {
	let state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
	let action = arguments[1];

	// TODO: remove subscribers, remove queued for windowName, remove stored for windowName, goodbye view

	switch (action.type) {

		//		actionSubscribers: {
		//			action: ['windowName']
		//		},
		// Doesn't handle action type filters, only array of action types
		case CONSTANTS.ADD_ACTION_SUBSCRIBER:
			{
				var _action$payload = action.payload;
				let windowName = _action$payload.windowName;
				let actionTypes = _action$payload.actionTypes;
				let updateSubscription = _action$payload.updateSubscription;

				// Convert pure arrays to List

				if (!_immutable2.default.List.isList(actionTypes)) {
					actionTypes = _immutable2.default.fromJS(actionTypes);
				}

				// Clear current action subscriptions from window unless the subscription is being updated
				if (updateSubscription !== true) {
					state = clearActionSubscriptionFromWindow(state, windowName);
				}

				return state.updateIn(['actionSubscribers'], _immutable2.default.Map(), actionSubscribers => {
					// Change {windowName, [actions] into action: [..., windowName]
					actionTypes.forEach(actionType => {
						actionSubscribers = actionSubscribers.updateIn([actionType], _immutable2.default.List(), actions => {
							// Don't subscribe multiple times
							if (!actions.contains(windowName)) {
								return actions.push(windowName);
							}

							return actions;
						});
					});

					return actionSubscribers;
				});
			}

		case CONSTANTS.REMOVE_ACTION_SUBSCRIBER:
			{
				var _action$payload2 = action.payload;
				const windowName = _action$payload2.windowName;
				const actionTypes = _action$payload2.actionTypes;


				return clearActionSubscriptionFromWindow(state, windowName);
			}

		case CONSTANTS.ADD_STATE_SUBSCRIBER:
			{
				const newStateSubscriber = _immutable2.default.fromJS(action.payload);
				const stateFilterShape = (0, _calculateStateFilterShape.calculateStateFilterShape)(state, newStateSubscriber.get('filter'));

				return state.setIn(['stateSubscribers', action.payload.windowName], newStateSubscriber).set('stateFilterShape', stateFilterShape);
			}

		case CONSTANTS.REMOVE_STATE_SUBSCRIBER:
			{
				const windowName = action.payload.windowName;


				let updatedState = state.removeIn(['stateSubscribers', windowName]);
				const stateFilterShape = (0, _calculateStateFilterShape.calculateStateFilterShape)(updatedState);

				return updatedState.set('stateFilterShape', stateFilterShape);
			}

		case CONSTANTS.CLEAR_SUBSCRIPTIONS_FOR_WINDOW:
		case CONSTANTS.ANNOUNCE_NEW_VIEW:
		case CONSTANTS.ANNOUNCE_CLOSING_VIEW:
			{
				const viewName = action.payload.viewName;


				return clearSubscriptionsFromWindow(state, viewName);
			}

		default:
			return state;
	}
}

function clearSubscriptionsFromWindow(state, windowName) {
	let updatedState = state.removeIn(['stateSubscribers', windowName]);
	const stateFilterShape = (0, _calculateStateFilterShape.calculateStateFilterShape)(updatedState);

	updatedState = clearActionSubscriptionFromWindow(updatedState, windowName);

	// Clear all subscribers
	return updatedState.set('stateFilterShape', stateFilterShape);
}

function clearActionSubscriptionFromWindow(state, windowName) {
	return state.updateIn(['actionSubscribers'], _immutable2.default.Map(), actions => {
		actions = actions.map((actionSubscribers, actionType) => {
			return actionSubscribers.filterNot(subscriber => subscriber === windowName);
		});

		actions = actions.filter(actionSubscribers => actionSubscribers.size > 0);

		return actions;
	});
}