'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.subscriptionsReducer = subscriptionsReducer;
exports.clearSubscriptionsForWindow = clearSubscriptionsForWindow;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _subscribersReducer = require('./subscribersReducer');

var _isBrowserProcess = require('../utils/isBrowserProcess');

var _utils = require('../utils');

var _actionTypes = require('../constants/actionTypes');

var CONSTANTS = _interopRequireWildcard(_actionTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const initialState = _immutable2.default.fromJS({

	// 		// our subscriptions we made to others
	// 		actionSubscriptions: {
	// 			windowName: {
	// 				actionTypes : ['actionType'],
	// 				isSubscribed: false,
	// 				requestSent : false,
	// 			}
	// 		},
	//
	// 		stateSubscriptions: {
	// 			windowName: {
	// 				windowName  : 'windowName',
	// 				filter      : 'filter',
	// 				isSubscribed: false,
	// 				requestSent : false,
	// 			}
	// 		},
});

function subscriptionsReducer() {
	let state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
	let action = arguments[1];


	switch (action.type) {
		case CONSTANTS.SUBSCRIBE_TO_STATE:
			{
				var _action$payload = action.payload;
				const viewName = _action$payload.viewName;
				const stateFilter = _action$payload.stateFilter;


				const subscription = _immutable2.default.fromJS({
					viewName,
					stateFilter,
					isSubscribed: false,
					requestSent: false
				});

				return state.setIn(['stateSubscriptions', viewName], subscription);
			}

		case CONSTANTS.UNSUBSCRIBE_FROM_STATE:
			{
				const viewName = action.payload.viewName;


				return state.deleteIn(['stateSubscriptions', viewName]);
			}

		case CONSTANTS.STATE_SUBSCRIPTION_STATUS:
			{
				var _action$payload2 = action.payload;
				const targetViewName = _action$payload2.targetViewName;
				const operation = _action$payload2.operation;
				const status = _action$payload2.status;


				if (operation === 'add') {
					return state.setIn(['stateSubscriptions', targetViewName, 'isSubscribed'], status === 'success');
				}

				return state;
			}

		case CONSTANTS.STATE_SUBSCRIPTION_REQUEST_SENT:
			{
				var _action$payload3 = action.payload;
				const targetViewName = _action$payload3.targetViewName;
				const operation = _action$payload3.operation;


				if (operation === 'add') {
					return state.setIn(['stateSubscriptions', targetViewName, 'requestSent'], true);
				}

				return state;
			}

		case CONSTANTS.SUBSCRIBE_TO_ACTIONS:
			{
				var _action$payload4 = action.payload;
				let viewName = _action$payload4.viewName;
				let actionTypes = _action$payload4.actionTypes;
				let updateSubscription = _action$payload4.updateSubscription;

				// Convert pure arrays to List

				if (!_immutable2.default.List.isList(actionTypes)) {
					actionTypes = _immutable2.default.fromJS(actionTypes);
				}

				// Clear current action subscription for window unless the subscription is being updated
				if (updateSubscription !== true) {
					state = state.deleteIn(['actionSubscriptions', viewName]);
				}

				return state.setIn(['actionSubscriptions', viewName, 'requestSent'], false).setIn(['actionSubscriptions', viewName, 'isSubscribed'], false).updateIn(['actionSubscriptions', viewName, 'actionTypes'], _immutable2.default.List(), actionsSubscriptions => {
					actionTypes.forEach(actionType => {
						if (!actionsSubscriptions.contains(actionType)) {
							actionsSubscriptions = actionsSubscriptions.push(actionType);
						}
					});

					return actionsSubscriptions;
				});
			}

		case CONSTANTS.UNSUBSCRIBE_FROM_ACTIONS:
			{
				var _action$payload5 = action.payload;
				const viewName = _action$payload5.viewName;
				const actionTypes = _action$payload5.actionTypes;

				// If no action types were given delete whole view key

				if (actionTypes == null) {
					return state.deleteIn(['actionSubscriptions', viewName]);
				}
				// Otherwise delete only given action types
				else {
						return state.updateIn(['actionSubscriptions', viewName], _immutable2.default.List(), actions => {
							return actions.filterNot(actionType => _lodash2.default.includes(actionTypes, actionType));
						});
					}
			}

		case CONSTANTS.ACTIONS_SUBSCRIPTION_STATUS:
			{
				var _action$payload6 = action.payload;
				const targetViewName = _action$payload6.targetViewName;
				const operation = _action$payload6.operation;
				const status = _action$payload6.status;


				if (operation === 'add') {
					return state.setIn(['actionSubscriptions', targetViewName, 'isSubscribed'], status === 'success');
				}

				return state;
			}

		case CONSTANTS.ACTIONS_SUBSCRIPTION_REQUEST_SENT:
			{
				var _action$payload7 = action.payload;
				const targetViewName = _action$payload7.targetViewName;
				const operation = _action$payload7.operation;


				if (operation === 'add') {
					return state.setIn(['actionSubscriptions', targetViewName, 'requestSent'], true);
				}

				return state;
			}

		case CONSTANTS.CLEAR_SUBSCRIPTIONS_FOR_WINDOW:
		case CONSTANTS.ANNOUNCE_NEW_VIEW:
		case CONSTANTS.ANNOUNCE_CLOSING_VIEW:
			{
				const viewName = action.payload.viewName;


				return clearSubscriptionsForWindow(state, viewName);
			}

		default:
			return state;
	}
}

// Keep subscriptions but remove indicator that we are subscribed
function clearSubscriptionsForWindow(state, windowName) {
	// Don't clear if we have no subscription for the window
	if (state.hasIn(['actionSubscriptions', windowName])) {
		state = state.setIn(['actionSubscriptions', windowName, 'isSubscribed'], false).setIn(['actionSubscriptions', windowName, 'requestSent'], false);
	}

	if (state.hasIn(['stateSubscriptions', windowName])) {
		state = state.setIn(['stateSubscriptions', windowName, 'isSubscribed'], false).setIn(['stateSubscriptions', windowName, 'requestSent'], false);
	}

	return state;
}