'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.ActionHandler = ActionHandler;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _selectors = require('../selectors');

var _actions = require('../actions');

var _utils = require('../utils');

function* ActionHandler() {
	yield* (0, _reduxSaga.takeEvery)('*', handleAction);
}

function* handleAction(action) {
	const actionSubscribers = yield (0, _effects.select)(_selectors.getActionSubscribers);

	if (!actionSubscribers.has(action.type)) {
		return;
	}

	const actionTypeSubscribers = yield actionSubscribers.get(action.type);

	// Dispatch action to actionTypeSubscriber window
	for (var i = 0; i < actionTypeSubscribers.size; i++) {
		const actionTypeSubscriber = yield actionTypeSubscribers.get(i);

		// Don't wait before dispatching to another subscriber
		yield (0, _effects.fork)(dispatchActionToSubscriber, action, actionTypeSubscriber);
	}
}

function* dispatchActionToSubscriber(action, subscriber) {
	yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)(action, subscriber));
}