'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.ClosingViewHandler = ClosingViewHandler;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _utils = require('../utils');

var _resendActionFromRendererToAllOtherRenderers = require('./resendActionFromRendererToAllOtherRenderers');

var _actionTypes = require('../constants/actionTypes');

var _variables = require('../variables');

var _updateGlobalWindowsInfo = require('./updateGlobalWindowsInfo');

function* ClosingViewHandler() {
	if (!(0, _utils.isBrowserProcess)()) {
		return;
	}

	yield* (0, _reduxSaga.takeEvery)(_actionTypes.ANNOUNCE_CLOSING_VIEW, handleClosingView);
}

// Note: The announced view should be in store by now.
function* handleClosingView(action) {
	const viewName = action.payload.viewName;


	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[CLOSED WINDOW]: "${ viewName }".`);
		(0, _utils.logToFile)(`[CLOSED WINDOW]: "${ viewName }".`);
	}

	yield (0, _updateGlobalWindowsInfo.updateGlobalWindowsInfo)();

	// Announce the closed window to all others
	yield (0, _effects.fork)(_resendActionFromRendererToAllOtherRenderers.resendActionFromRendererToAllOtherRenderers, action, viewName);
}