'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.NewViewHandler = NewViewHandler;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _selectors = require('../selectors');

var _actions = require('../actions');

var _utils = require('../utils');

var _resendActionFromRendererToAllOtherRenderers = require('./resendActionFromRendererToAllOtherRenderers');

var _announceAllWindowsToWindow = require('./announceAllWindowsToWindow');

var _actionTypes = require('../constants/actionTypes');

var _SubscriptionsHandler = require('./SubscriptionsHandler');

var _variables = require('../variables');

var _updateGlobalWindowsInfo = require('./updateGlobalWindowsInfo');

function* NewViewHandler() {
	yield* (0, _reduxSaga.takeEvery)(_actionTypes.ANNOUNCE_NEW_VIEW, handleNewView);
}

// Note: The announced view should be in store by now.
function* handleNewView(action) {
	const viewName = action.payload.viewName;


	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[NEW WINDOW]: ${ viewName }`);
		(0, _utils.logToFile)(`[NEW WINDOW]: ${ viewName }`);
	}

	if ((0, _utils.isBrowserProcess)()) {
		yield (0, _updateGlobalWindowsInfo.updateGlobalWindowsInfo)();

		try {
			let window = (0, _utils.getWindowFromName)(viewName);
			window.on('closed', () => {
				(0, _variables.dispatch)((0, _actions.announceClosingView)(viewName));
			});
		} catch (e) {
			(0, _utils.logToFile)(`ERROR: ${ e.name }: ${ e.message }`);
			console.warn(`Subscribing to closing event for window "${ viewName }" was unsuccessful. You need to need to announce the close event manually`);
		}

		// New window announces itself only to the browser process, so we need to resend the action to all renderers
		yield [
		// Announce browser to the new window
		(0, _effects.fork)(announceBrowserToRenderer, viewName),
		// Announce the new window to all others
		(0, _effects.fork)(_resendActionFromRendererToAllOtherRenderers.resendActionFromRendererToAllOtherRenderers, action, viewName),
		// Announce all others to the new window
		(0, _effects.fork)(_announceAllWindowsToWindow.announceAllWindowsToWindow, viewName)];
	}

	// Dispatch our subscription for the new window
	yield dispatchSubscriptions(action);
}

function* dispatchSubscriptions(action) {
	const targetViewName = action.payload.viewName;


	yield [(0, _effects.fork)(_SubscriptionsHandler.handleStateSubscribe, targetViewName), (0, _effects.fork)(_SubscriptionsHandler.handleActionsSubscribe, targetViewName)];
}

function* announceBrowserToRenderer(windowName) {
	const browserName = yield (0, _effects.select)(_selectors.getViewName);
	yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.announceNewView)(browserName), windowName));
}

// function* dispatchStoredAndQueuedActions(action)
// {
// 	const {viewName} = action.payload;
//
// 	// Stored go first, since they are more important (add subscriber needs to be before any action)
// 	const storedActions = yield select(getStoredActionsForWindow, viewName);
// 	const queuedActions = yield select(getQueuedActionsForWindow, viewName);
//
// 	// Dispatch stored actions
// 	for (var i = 0; i < storedActions.size; i++)
// 	{
// 		const storedAction = yield storedActions.get(i);
// 		yield sagaPut(storedAction);
// 	}
//
// 	// Dispatch queued actions
// 	for (var i = 0; i < queuedActions.size; i++)
// 	{
// 		const queuedAction = yield queuedActions.get(i);
// 		yield sagaPut(queuedAction);
// 	}
//
// 	// Clear queue
// 	if (queuedActions.size > 0)
// 	{
// 		yield sagaPut(clearActionQueueForView(viewName));
// 	}
// }