'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.StateChangeHandler = StateChangeHandler;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _actionTypes = require('../constants/actionTypes');

var _utils = require('../utils');

var _selectors = require('../selectors');

var _actions = require('../actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function* StateChangeHandler() {
	yield* (0, _reduxSaga.takeEvery)(_actionTypes.STATE_CHANGED, handleStateChange);
}

function* handleStateChange(action) {
	// Don't diff state changes if
	// 1) We have no state subscribers
	// 2) Nothing in the subscribed state changed

	const stateSubscribers = yield (0, _effects.select)(_selectors.getStateSubscribers);

	// 1)
	if (stateSubscribers.size === 0) {
		return;
	}

	const stateFilterShape = yield (0, _effects.select)(_selectors.getStateFilterShape);
	// 2)

	// test each reducer if changed
	// Doesn't take into account that a reducer might be removed or added
	// Quick check if anything changed
	let hasChanged = false;
	stateFilterShape.forEach((reducer, key) => {
		if (action.payload.prevState[key] !== action.payload.nextState[key]) {
			hasChanged = true;
			return false;
		}
	});

	if (!hasChanged) {
		return;
	}

	// Using while since we can't use yield from forEach function
	let stateSubscriber, stateDiff, filter;
	var stateSubscribersIterator = stateSubscribers.values();
	while (!(stateSubscriber = stateSubscribersIterator.next()).done) {
		stateDiff = yield _immutable2.default.Map();
		filter = yield stateSubscriber.value.get('filter');

		stateDiff = (0, _utils.diffStateWithShape)(action.payload.prevState, action.payload.nextState, filter);

		if (stateDiff.size > 0) {
			const windowName = yield stateSubscriber.value.get('windowName');
			yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.stateUpdate)(stateDiff.toJS()), windowName));
		}
	}
}