'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.SubscribersHandler = SubscribersHandler;
exports.handleAddedStateSubscriber = handleAddedStateSubscriber;
exports.handleRemovedStateSubscriber = handleRemovedStateSubscriber;
exports.handleAddedActionsSubscriber = handleAddedActionsSubscriber;
exports.handleRemovedActionsSubscriber = handleRemovedActionsSubscriber;

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _actionTypes = require('../constants/actionTypes');

var _selectors = require('../selectors');

var _index = require('../utils/index');

var _utils = require('../utils');

var _index2 = require('../actions/index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function* SubscribersHandler() {
	yield* (0, _reduxSaga.takeEvery)([_actionTypes.ADD_STATE_SUBSCRIBER, _actionTypes.ADD_ACTION_SUBSCRIBER, _actionTypes.REMOVE_STATE_SUBSCRIBER, _actionTypes.REMOVE_ACTION_SUBSCRIBER], handleSubscribers);
}

function* handleSubscribers(action) {
	switch (action.type) {
		case _actionTypes.ADD_STATE_SUBSCRIBER:
			{
				yield handleAddedStateSubscriber(action);
				break;
			}
		case _actionTypes.REMOVE_STATE_SUBSCRIBER:
			{
				yield handleRemovedStateSubscriber(action);
				break;
			}
		case _actionTypes.ADD_ACTION_SUBSCRIBER:
			{
				yield handleAddedActionsSubscriber(action);
				break;
			}
		case _actionTypes.REMOVE_ACTION_SUBSCRIBER:
			{
				yield handleRemovedActionsSubscriber(action);
				break;
			}
	}
}

// send full state to new subscriber
function* handleAddedStateSubscriber(action) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);
	const targetViewName = yield action.payload.windowName;

	yield (0, _utils.sagaPut)((0, _index2.makeGlobalAction)((0, _index2.stateSubscriptionStatus)(currentViewName, 'add', 'success'), targetViewName));

	// Set initial subscribed state
	const stateSubscriber = yield (0, _effects.select)(_selectors.getStateSubscriberFromName, { name: targetViewName });
	const filteredState = yield (0, _effects.select)(_index.filterStateToShape, stateSubscriber.get('filter', _immutable2.default.Map()));

	yield (0, _utils.sagaPut)((0, _index2.makeGlobalAction)((0, _index2.setInitialSubscribedState)(filteredState.toJS(), currentViewName), targetViewName));
}

function* handleRemovedStateSubscriber(action) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);
	const targetViewName = yield action.payload.windowName;

	yield (0, _utils.sagaPut)((0, _index2.makeGlobalAction)((0, _index2.stateSubscriptionStatus)(currentViewName, 'remove', 'success'), targetViewName));
}

function* handleAddedActionsSubscriber(action) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);
	const targetViewName = yield action.payload.windowName;

	yield (0, _utils.sagaPut)((0, _index2.makeGlobalAction)((0, _index2.actionsSubscriptionStatus)(currentViewName, 'add', 'success'), targetViewName));
}

function* handleRemovedActionsSubscriber(action) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);
	const targetViewName = yield action.payload.windowName;

	yield (0, _utils.sagaPut)((0, _index2.makeGlobalAction)((0, _index2.actionsSubscriptionStatus)(currentViewName, 'remove', 'success'), targetViewName));
}