'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.SubscriptionsHandler = SubscriptionsHandler;
exports.handleActionsSubscribe = handleActionsSubscribe;
exports.handleActionsUnsubscribe = handleActionsUnsubscribe;
exports.handleStateSubscribe = handleStateSubscribe;
exports.handleStateUnsubscribe = handleStateUnsubscribe;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _libraryConstants = require('../constants/libraryConstants');

var _utils = require('../utils');

var _actions = require('../actions');

var _actionTypes = require('../constants/actionTypes');

var _selectors = require('../selectors');

var _logToFile = require('../utils/logToFile');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function* SubscriptionsHandler() {
	yield* (0, _reduxSaga.takeEvery)([_actionTypes.SUBSCRIBE_TO_STATE, _actionTypes.SUBSCRIBE_TO_ACTIONS, _actionTypes.UNSUBSCRIBE_FROM_ACTIONS, _actionTypes.UNSUBSCRIBE_FROM_STATE], handleSubscription);
}

function* handleSubscription(action) {
	switch (action.type) {
		case _actionTypes.SUBSCRIBE_TO_STATE:
			{
				var _action$payload = action.payload;
				const viewName = _action$payload.viewName;
				const stateFilter = _action$payload.stateFilter;

				yield handleStateSubscribe(viewName, stateFilter);
				break;
			}
		case _actionTypes.UNSUBSCRIBE_FROM_STATE:
			{
				const viewName = action.payload.viewName;

				yield handleStateUnsubscribe(viewName);
				break;
			}
		case _actionTypes.SUBSCRIBE_TO_ACTIONS:
			{
				var _action$payload2 = action.payload;
				const viewName = _action$payload2.viewName;
				const actionTypes = _action$payload2.actionTypes;

				yield handleActionsSubscribe(viewName, actionTypes);
				break;
			}
		case _actionTypes.UNSUBSCRIBE_FROM_ACTIONS:
			{
				var _action$payload3 = action.payload;
				const viewName = _action$payload3.viewName;
				const actionTypes = _action$payload3.actionTypes;

				yield handleActionsUnsubscribe(viewName, actionTypes);
				break;
			}
	}
}

// ACTIONS
function* handleActionsSubscribe(targetViewName, actionTypes) {
	const actionSubscriptions = yield (0, _effects.select)(_selectors.getActionsSubscriptionForWindow, targetViewName);
	if (!_immutable2.default.Map.isMap(actionSubscriptions) || actionSubscriptions.size === 0 || !(0, _utils.isTargetAvailable)(targetViewName)) {
		return;
	}

	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);

	if (actionSubscriptions.has('actionTypes') && actionSubscriptions.get('isSubscribed') !== true && actionSubscriptions.get('requestSent') !== true) {
		yield (0, _utils.sagaPut)((0, _actions.actionsSubscriptionRequestSent)(targetViewName, 'add'));
		yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.addActionSubscriber)(currentViewName, actionSubscriptions.get('actionTypes').toJS()), targetViewName));
	}
}

function* handleActionsUnsubscribe(targetViewName, actionTypes) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);

	yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.removeActionsSubscriber)(currentViewName, actionTypes), targetViewName));
}

// STATE
function* handleStateSubscribe(targetViewName, stateFilter) {
	const stateSubscription = yield (0, _effects.select)(_selectors.getStateSubscriptionForWindow, targetViewName);

	if (!_immutable2.default.Map.isMap(stateSubscription) || stateSubscription.size === 0 || !(0, _utils.isTargetAvailable)(targetViewName)) {
		return;
	}

	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);

	if (stateSubscription.has('stateFilter') && stateSubscription.get('isSubscribed') !== true && stateSubscription.get('requestSent') !== true) {
		yield (0, _utils.sagaPut)((0, _actions.stateSubscriptionRequestSent)(targetViewName, 'add'));
		yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.addStateSubscriber)(currentViewName, stateSubscription.get('stateFilter').toJS()), targetViewName));
	}
}

function* handleStateUnsubscribe(targetViewName) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);

	yield (0, _utils.sagaPut)((0, _actions.makeGlobalAction)((0, _actions.removeStateSubscriber)(currentViewName), targetViewName));
}