'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.filterMapToShape = filterMapToShape;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _isImmutableJsMap = require('./isImmutableJsMap');

var _libraryConstants = require('../constants/libraryConstants');

var _processShapeMapPath = require('./processShapeMapPath');

var _processShapeEditPath = require('./processShapeEditPath');

var _processShapeMapValues = require('./processShapeMapValues');

var _processFilterByValue = require('./processFilterByValue');

var _listInfo = require('./listInfo');

var ListInfo = _interopRequireWildcard(_listInfo);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO: Handle errors
function _filterToShape(mutable, obj, shape) {
	let path = arguments.length <= 3 || arguments[3] === undefined ? [] : arguments[3];
	let fullObj = arguments.length <= 4 || arguments[4] === undefined ? null : arguments[4];

	if (fullObj === null) {
		fullObj = obj;
	}

	// Filter object (Complex filter operation)
	if ((0, _isImmutableJsMap.isImmutableJsMap)(shape) && shape.has(_libraryConstants.SHAPE_FILTER_KEY) && shape.get(_libraryConstants.SHAPE_FILTER_KEY) === true) {
		processFilterObject(mutable, shape, obj, path, fullObj);

		return;
	}

	shape.forEach((value, key) => {
		// TRUE (Get full key subtree)
		if (value === true) {
			if (obj.has(key)) {
				mutable.setIn(path.concat(key), obj.get(key));
			}
		}
		// STRING (Edit path)
		else if (typeof value === 'string') {
				if (obj.has(key)) {
					let editedPath = (0, _processShapeEditPath.processShapeEditPath)(value, path.concat(key));

					if (editedPath.size <= 0) {
						return;
					}

					mutable.setIn(editedPath, obj.get(key));
				}
			}
			// LIST (Select keys)
			else if (_immutable2.default.List.isList(value)) {
					processList(mutable, key, value, obj, path, fullObj);
				}
				// MAP (Go deeper)
				else // filter key
					{
						// Consider maps only
						if ((0, _isImmutableJsMap.isImmutableJsMap)(value)) {
							if (obj.has(key)) {
								// Nested filter
								_filterToShape(mutable, obj.get(key), shape.get(key), (0, _cloneDeep2.default)(path.concat(key)), fullObj);
							}
						}
					}
	});
}

function processFilterObject(mutable, shape, obj, path, fullObj) {
	// Edit path
	let newPath = path;
	if (shape.has('editPath') && typeof shape.get('editPath') === 'string') {
		newPath = (0, _processShapeEditPath.processShapeEditPath)(shape.get('editPath'), path);
	}

	// Slice list
	let newObj = obj;
	if (shape.has('sliceList')) {
		if (_immutable2.default.List.isList(obj)) {
			newObj = obj.slice(...shape.get('sliceList'));
		}
	}

	// Select keys (only if the obj is a Map)
	if (shape.has('selectKeys') && _immutable2.default.Map.isMap(newObj) && _immutable2.default.List.isList(shape.get('selectKeys'))) {
		const selectRoot = shape.get('selectRoot', _immutable2.default.List());

		let selectedPaths = (0, _processShapeMapPath.processShapeMapPath)(shape.get('selectKeys'), fullObj, selectRoot);

		newObj = newObj.clear().withMutations(objMutable => {
			selectedPaths.forEach(selectedPath => {
				if (!_immutable2.default.List.isList(selectedPath)) {
					selectedPath = _immutable2.default.List([selectedPath]);
				}

				if (newObj.hasIn(selectedPath)) {
					objMutable.setIn(selectedPath, newObj.getIn(selectedPath));
				}
			});
		});
	}

	if (shape.has('filterByValue') && _immutable2.default.Map.isMap(shape.get('filterByValue'))) {
		newObj = (0, _processFilterByValue.processFilterByValue)(newObj, shape.get('filterByValue'), fullObj);
	}

	// Filter keys
	if (shape.has('filterKeys') && (_immutable2.default.Map.isMap(shape.get('filterKeys')) || _immutable2.default.List.isList(shape.get('filterKeys')))) {
		newObj = (0, _processShapeMapValues.processShapeMapValues)(newObj, shape.get('filterKeys'));
	}

	// Set filtered value
	mutable.setIn(newPath, newObj);
}

function processList(mutable, key, value, obj, path, fullObj) {

	// Slice list
	if (ListInfo.isSliceList(value)) {
		// Slice params [0,1], [-1]
		if (obj.has(key)) {
			let got = obj.get(key);
			if (_immutable2.default.List.isList(got)) {
				mutable.setIn(path.concat(key), got.slice(...value));
			}
		}
	}
	// 	// List of maps [{}, {}, {}]
	// 	else if (value.size >= 1 && ListInfo.isMapsOnlyList(value))
	// 	{}
	// Select keys
	else if (value.size >= 1) {
			var processedPath = (0, _processShapeMapPath.processShapeMapPath)(value, fullObj);

			processedPath.forEach(processed => {
				// Make list for setIn/getIn, that needs list as a path
				if (!_immutable2.default.List.isList(processed)) {
					processed = _immutable2.default.List([processed]);
				}

				if (obj.hasIn([key, ...processed])) {
					mutable.setIn([...path, key, ...processed], obj.getIn([key, ...processed]));
				}
			});
		}
}

function filterMapToShape(map, shape) {
	// Return full object if filter is true
	if (shape == null || shape === true) {
		// QQQ: clone?
		return map;
	}

	let filteredMap = _immutable2.default.Map();

	// Return null for invalid arguments
	if (!_immutable2.default.Map.isMap(map) || !_immutable2.default.Map.isMap(shape)) {
		return null;
	}

	filteredMap = filteredMap.withMutations(function (mutable) {
		_filterToShape(mutable, map, shape);
	});

	return filteredMap;
}