'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.mergeWithShape = mergeWithShape;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _isImmutableJsMap = require('./isImmutableJsMap');

var _libraryConstants = require('../constants/libraryConstants');

var _processShapeEditPath = require('./processShapeEditPath');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _mergeWithShape(mutable, mergingObj, shape) {
	let path = arguments.length <= 3 || arguments[3] === undefined ? [] : arguments[3];
	let fullMergingObj = arguments.length <= 4 || arguments[4] === undefined ? null : arguments[4];

	if (fullMergingObj === null) {
		fullMergingObj = mergingObj;
	}

	shape.forEach((value, key) => {
		// Get full key tree
		// Treats any shape filter
		if (value === true || typeof value === 'string' || _immutable2.default.List.isList(value)) {
			if (mergingObj.has(key)) {
				mutable.setIn(path.concat(key), mergingObj.get(key));
			}
		}
		// filter key
		else if ((0, _isImmutableJsMap.isImmutableJsMap)(value)) {
				// Filter object
				if (value.has(_libraryConstants.SHAPE_FILTER_KEY)) {
					// Don't do anything unless the `filter` key is true
					if (value.get(_libraryConstants.SHAPE_FILTER_KEY) !== true) {
						return;
					}

					// If the `filter` contains editPath we must use it
					if (value.has('editPath')) {
						const editedPath = (0, _processShapeEditPath.processShapeEditPath)(value.get('editPath', ''), path);
						mutable.setIn(editedPath, fullMergingObj.getIn(editedPath));
					}
					// if there is no editPath, then set the value (same as true above)
					else {
							mutable.setIn(path.concat(key), mergingObj.get(key));
						}
				} else {
					_mergeWithShape(mutable, mergingObj.get(key), shape.get(key), (0, _cloneDeep2.default)(path.concat(key)), fullMergingObj);
				}
			}
	});
}

function mergeWithShape(map, mergingMap, shape) {
	// Return full object if filter is true
	if (shape === true) {
		// TODO: clone?
		return map;
	}

	map = map.withMutations(function (mutable) {
		_mergeWithShape(mutable, mergingMap, shape);
	});

	return map;
}